package edu.kit.ipd.descartes.adaptation.model.perfdatarepo;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.Iterator;

import junit.framework.TestCase;

import org.junit.Before;
import org.junit.Test;

import edu.kit.ipd.descartes.perfdatarepo.MetricValue;
import edu.kit.ipd.descartes.ql.models.mapping.domain.DecimalResult;
import edu.kit.ipd.descartes.ql.models.mapping.domain.DomainFactory;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Entity;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Resource;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.MappingFactory;

public class PerfDataRepoConverterTest extends TestCase {
    
    Resource r1 = DomainFactory.eINSTANCE.createResource();
    Resource r2 = DomainFactory.eINSTANCE.createResource();
    
    DecimalResult p1 = DomainFactory.eINSTANCE.createDecimalResult();
    DecimalResult p2 = DomainFactory.eINSTANCE.createDecimalResult();
    DecimalResult p3 = DomainFactory.eINSTANCE.createDecimalResult();

    EntityMapping m = MappingFactory.eINSTANCE.createEntityMapping();

    @Before
    protected void setUp() throws Exception {
        super.setUp();
        
        p1.setAccuracy(new BigDecimal(0.0));
        p1.setMetricName("ResponseTimeMetricNameTest");
        p1.setValid(Boolean.TRUE);
        p1.setValue(new BigDecimal(100.0));
        
        p2.setAccuracy(new BigDecimal(0.0));
        p2.setMetricName("ResponseTimeMetricNameTest");
        p2.setValid(Boolean.TRUE);
        p2.setValue(new BigDecimal(110.0));
        
        p3.setAccuracy(new BigDecimal(0.0));
        p3.setMetricName("ResponseTimeMetricNameTest");
        p3.setValid(Boolean.TRUE);
        p3.setValue(new BigDecimal(100.0));
        
        r1.setAlias("Resource1");
        r1.setIdentifier("Resource1-ID");
        r1.getProbes().add(p1);
        r1.getProbes().add(p2);
        
        r2.setAlias("Resource2");
        r2.setIdentifier("Resource2-ID");
        r2.getProbes().add(p3);
        
        m.getResources().add(r1);
        m.getResources().add(r2);
    }

    @Test
    public void testConvertMapping() {
        Collection<MetricValue> result = PerfDataRepoConverter.convertMapping(m);
        assertTrue(result.size() == calcResultSize(m));
        Iterator<MetricValue> i = result.iterator();
        MetricValue v1 = i.next();
        MetricValue v2 = i.next();
        assertEquals(v1.getMetricType(), v2.getMetricType());
    }

    private long calcResultSize(EntityMapping mapping) {
        long resultsize = 0;
        for (Entity e : mapping.getResources()) {
            resultsize += e.getProbes().size();
        }
        for (Entity e : mapping.getServices()) {
            resultsize += e.getProbes().size();
        }
        return resultsize;
    }

}
